/*
 * flexcop-common.h
 * 
 * see flexcop.c for copyright information.
 *
 */
#ifndef __FLEXCOP_COMMON_H__
#define __FLEXCOP_COMMON_H__

#include "flexcop-reg.h"

#define MAX_N_HW_FILTERS	(6+32)
#define N_PID_SLOTS		256


/* Control structure for data definitions that are common to
 * the B2C2-based PCI and USB devices.
 */
struct flexcop_device {
#if 0
	u8 card_revision;
	u32 b2c2_revision;
	u32 dev_device; /* Either pdev or udev device */
	u32 dev_vendor; /* Either pdev or udev device */
	u32 dev_subsystem_device; /* Either pdev or udev device */
	u32 dev_subsystem_vendor; /* Either pdev or udev device */
	u32 pid_filter_max;
	u32 mac_filter_max;
	u8 mac_addr[8];
	u32 dw_sram_type
	struct dvb_demux demux;
	struct dmxdev dmxdev;
	struct dmx_frontend hw_frontend;
	struct dmx_frontend mem_frontend;
	
	
	struct dvb_net dvbnet;

	int capturing;

	int useable_hw_filters;
	u16 hw_pids[MAX_N_HW_FILTERS];
	u16 pid_list[N_PID_SLOTS];
	int pid_rc[N_PID_SLOTS];	/* ref counters for the pids */
	int pid_count;
	int whole_bandwidth_count;
	u32 mac_filter;

#endif
	/* general */
	struct device *dev; /* for firmware_class */

	/* device information */
	u8 mac_address[6];
	int extra_32_hw_pid_filter;
	flexcop_revision_t rev;
	flexcop_device_type_t dev_type;
	
	/* dvb stuff */	
	struct dvb_adapter *dvb_adapter;
	struct dvb_frontend *fe;
	int (*fe_sleep) (struct dvb_frontend *);
	
	struct i2c_adapter i2c_adap;
	struct semaphore i2c_sem;

	/* options and status */
	int feedcount;
	int pid_filtering;
	
	/* bus specific callbacks */
	flexcop_ibi_value (*read_ibi_reg)  (struct flexcop_device *, flexcop_ibi_register);
	int               (*write_ibi_reg) (struct flexcop_device *, flexcop_ibi_register, flexcop_ibi_value);
	

	int (*i2c_request) (struct flexcop_device*, flexcop_access_op_t, flexcop_i2c_port_t, u8 chipaddr, u8 addr, u8 *buf, u16 len);

//	void (*open_stream) (struct flexcop_device*, u16 pid);
//	void (*close_stream) (struct flexcop_device*, u16 pid);

	void *bus_specific;
};

/* Prototypes */
struct flexcop_device *flexcop_device_kmalloc(size_t bus_specific_len);
void flexcop_device_kfree(struct flexcop_device*);

int  flexcop_initialize(struct flexcop_device*);
void flexcop_uninitialize(struct flexcop_device*);

int flexcop_master_xfer(struct i2c_adapter*, struct i2c_msg[], int);
/* the PCI uses this a i2c_request callback, whereas the usb part has its own
 * one. We have it in flexcop-i2c.c, because it is going via the actual
 * I2C-channel of the flexcop. 
 */
int flexcop_i2c_request(struct flexcop_device*, flexcop_access_op_t,
		        flexcop_i2c_port_t, u8 chipaddr, u8 addr, u8 *buf, u16 len);

int flexcop_frontend_init(struct flexcop_device *card);

int flexcop_sram_set_dest(struct flexcop_device *fc, flexcop_sram_dest_t dest, flexcop_sram_dest_target_t target);

int flexcop_eeprom_check_mac_addr(struct flexcop_device *fc,int type);

void flexcop_determine_revision(struct flexcop_device *fc);

/*
void b2c2_flexcop_reset(struct flexcop_device *);
void b2c2_filter_enable_mask_filter(struct flexcop_device *card, u32 op);
int b2c2_add_pid(struct flexcop_device *card, u16 pid);
int b2c2_remove_pid(struct flexcop_device *card, u16 pid);
*/
#endif
